unit UBinTree;

interface
uses classes,UTreeNode;

type TBinTree = class(TObject)
       private
         Root        : TTreeNode;
         CurrentNode : TTreeNode;
       public
         constructor Create(Elem: TObject);
         destructor Destroy;                        override;
         procedure AddLeft  (Elem: TObject);
         procedure AddRight (Elem: TObject);
         procedure AddLeftTree  (bin: TBinTree);
         procedure AddRightTree (bin: TBinTree);

         procedure DeleteLeft;
         procedure DeleteRight;
         procedure reset;
         procedure Left;
         procedure Right;
         function  ReadLeft: TObject;
         function  ReadRight: TObject;
         function  ReadLeftTree: TBinTree;
         function  ReadRightTree: TBinTree;
         function  ReadCurrentTree: TBinTree;

         procedure WriteElement (Elem: TObject);
         function  ReadElement: TObject;
         function  ReadNode   :TTreeNode;
         function  ReadRoot   :TTreeNode;
         function  IsEmpty: Boolean;
         function  IsLeaf: Boolean;          
     end;

implementation

constructor TBinTree.Create(Elem: TObject);
begin
   Root:=TTreeNode.create(Elem);
   CurrentNode := Root;
end;

destructor TBinTree.Destroy;
var LeftTree, RightTree:TBinTree;
begin
  root.destroy;
  inherited destroy
end;

procedure TBinTree.AddLeft (Elem: TObject);
var new: TTreenode;
begin
   if not isempty then begin
      new:= TTreenode.create(Elem);
      CurrentNode.ReadLeftNode.Destroy;
      CurrentNode.WriteLeftNode(new)
   end
end;

procedure TBinTree.AddRight (Elem: TObject);
var new: TTreenode;
begin
   if not isempty then begin
      new:= TTreenode.create(Elem);
      CurrentNode.ReadRightNode.Destroy;
      CurrentNode.WriteRightNode(new)
   end   
end;

procedure TBinTree.AddLeftTree (Bin: TBinTree);
begin
   CurrentNode.ReadLeftNode.Destroy;
   CurrentNode.WriteLeftNode(Bin.ReadRoot)
end;

procedure TBinTree.AddRightTree (Bin: TBinTree);
begin
   CurrentNode.ReadRightNode.Destroy;
   CurrentNode.WriteRightNode(Bin.ReadRoot)
end;

procedure TBinTree.DeleteLeft;
begin
   CurrentNode.ReadLeftNode.destroy;
   CurrentNode.WriteLeftNode(TTreenode.CreateNil)
end;

procedure TBinTree.DeleteRight;
begin
   CurrentNode.ReadRightNode.destroy;
   CurrentNode.WriteRightNode(TTreeNode.CreateNil)
end;

procedure TBinTree.Reset;
begin
      CurrentNode := Root
end;

procedure TBinTree.Left;
begin
   if not (CurrentNode.ReadLeftNode.isempty) then
      CurrentNode := CurrentNode.ReadLeftNode
end;

procedure TBinTree.Right;
begin
   if not (CurrentNode.ReadRightNode.isempty) then
      CurrentNode := CurrentNode.ReadRightNode
end;

function  TBinTree.ReadLeft: TObject;
begin
   result := CurrentNode.ReadLeftNode.ReadValue
end;

function  TBinTree.ReadRight: TObject;
begin
   result := CurrentNode.ReadRightNode.ReadValue
end;

function TBinTree.ReadCurrentTree: TBinTree;
var newtree:TBinTree;
begin
   newtree.Create(nil);
   newtree.Root := currentNode;
   newtree.currentNode := newtree.Root;
   result:=newtree
end;

function TBinTree.ReadLeftTree: TBinTree;
var newtree:TBinTree;
begin
   newtree.Create(nil);
   newtree.Root := currentNode.ReadLeftNode;
   newtree.currentNode := newtree.Root;
   result:=newtree
end;

function TBinTree.ReadRightTree: TBinTree;
var newtree:TBinTree;
begin
   newtree.Create(nil);
   newtree.Root := currentNode.ReadRightNode;
   newtree.currentNode := newtree.Root;
   result:=newtree
end;

procedure TBinTree.WriteElement (Elem: TObject);
begin
   CurrentNode.WriteValue(Elem)
end;

function  TBinTree.ReadElement: TObject;
begin
   result := CurrentNode.ReadValue
end;

function  TBinTree.ReadRoot: TTreeNode;
begin
   result := Root
end;

function  TBinTree.ReadNode: TTreeNode;
begin
   result := currentNode
end;

function  TBinTree.IsEmpty: Boolean;
begin
   result := root.isempty
end;

function  TBinTree.IsLeaf: Boolean;
begin
   result := CurrentNode.IsLeaf
end;

end.


